@@ -39,6 +39,52 @@ PERCENTAGE = beye.DataSourceHal.PERCENTAGE
 CHARGE = beye.DataSourceHal.CHARGE
 VOLTAGE = beye.DataSourceHal.VOLTAGE
 
+colorSchemes = \
+{
+  'Default': {
+    'background': (0, 0, 0),
+    'grid': (40, 40, 45),
+    'x_labels': (120, 120, 120),
+    'data_percentage_1': (0, 255, 0),
+    'data_percentage_2': (0, 127, 0),
+    'data_voltage_1': (60, 60, 100),
+    'data_voltage_2': (30, 30, 50),
+    'data_charge_1': (0, 100, 20),
+    'data_charge_2': (0, 50, 10),
+    'data_charging': (35, 35, 0),
+    'data_full': (0, 50, 0)
+  },
+
+  'Default - High contrast': {
+    'background': (0, 0, 0),
+    'grid': (100, 100, 110),
+    'x_labels': (190, 190, 190),
+    'data_percentage_1': (80, 255, 80),
+    'data_percentage_2': (40, 200, 40),
+    'data_voltage_1': (120, 120, 200),
+    'data_voltage_2': (90, 90, 150),
+    'data_charge_1': (120, 170, 120),
+    'data_charge_2': (90, 100, 90),
+    'data_charging': (80, 80, 0),
+    'data_full': (0, 100, 0)
+  },
+
+  'White background': {
+    'background': (255, 255, 255),
+    'grid': (40, 40, 40),
+    'x_labels': (40, 40, 40),
+    'data_percentage_1': (0, 150, 0),
+    'data_percentage_2': (30, 200, 30),
+    'data_voltage_1': (110, 110, 190),
+    'data_voltage_2': (170, 170, 215),
+    'data_charge_1': (180, 210, 180),
+    'data_charge_2': (200, 255, 200),
+    'data_charging': (240, 240, 110),
+    'data_full': (130, 190, 130)
+  }
+
+}
+
 class MainWindow(hildon.StackableWindow):
     def __init__(self, dataStorage):
         hildon.StackableWindow.__init__(self)
@@ -56,7 +102,12 @@ class MainWindow(hildon.StackableWindow):
         self.layout = gtk.Fixed()
         self.add(self.layout)
 
-        self.graph = Graph(self.dataStorage)
+        try:
+            self.colorScheme = colorSchemes[self.dataStorage.getConfig('display:colorScheme', 'Default')]
+        except KeyError:
+            self.colorScheme = colorSchemes['Default']
+
+        self.graph = Graph(self.dataStorage, self.colorScheme)
         self.layout.put(self.graph, 0, 0)
 
         gtk.set_application_name("battery-eye")
@@ -71,12 +122,12 @@ class MainWindow(hildon.StackableWindow):
     def setupMenu(self):
         self.menu = hildon.AppMenu()
         
-        for type, label, default in ((PERCENTAGE, '%', 1), (CHARGE, 'mAh', 0), (VOLTAGE, 'mV', 0)):
+        for type, label, default in ((PERCENTAGE, '%', 1), (CHARGE, 'mAh', 1), (VOLTAGE, 'mV', 1)):
             f = hildon.GtkToggleButton(gtk.HILDON_SIZE_AUTO)
             f.set_label(label)
             f.set_mode(False)
             f.set_active(bool(int(self.dataStorage.getConfig('display:' + type, default))))
-            f.set_image(self.createLegendImage(self.graph.dataColors[type], 24))
+            f.set_image(self.createLegendImage(self.graph.dataColors[type][0], 24))
             #f.connect("clicked", lambda button: self.changeDataset(button, type))
             f.connect("clicked", self.changeDataset, type)
             self.menu.add_filter(f)
@@ -98,6 +149,12 @@ class MainWindow(hildon.StackableWindow):
 
         self.menu.append(self.deleteButton)
 
+        self.settingsButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.settingsButton.set_label('Settings')
+        self.settingsButton.connect("clicked", self.showSettings)
+
+        self.menu.append(self.settingsButton)
+
         #self.incUpd = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
         #self.incUpd.set_label('Incremental upd.')
         #self.incUpd.connect("clicked", lambda x: self.graph.updateIncremental())
@@ -113,6 +170,16 @@ class MainWindow(hildon.StackableWindow):
         self.menu.show_all()
         self.set_app_menu(self.menu)
 
+    def showSettings(self, widget):
+        dlg = SettingsDialog(self)
+        response = dlg.showSettings(self)
+        if response == gtk.RESPONSE_OK:
+            print "Saving settings"
+            dlg.applySettings(self)
+        else:
+            print "Settings dialog canceled"
+        dlg.destroy()
+
     def changeDataset(self, button, dataset):
         if button.get_active():
             self.graph.enableDataset(dataset)
@@ -122,11 +189,12 @@ class MainWindow(hildon.StackableWindow):
 
     def focusChanged(self, window, property):
         if self.get_is_topmost():
-            print "\n%s Got focus" % str(datetime.datetime.now())
+            #print "\n%s Got focus" % str(datetime.datetime.now())
             if self.updateRequired:
                 self.queueUpdate()
         else:
-            print "\n%s Lost focus" % str(datetime.datetime.now())
+            pass
+            #print "\n%s Lost focus" % str(datetime.datetime.now())
         return False
 
     def deleteData(self, widget):
@@ -134,13 +202,22 @@ class MainWindow(hildon.StackableWindow):
         dlg.set_title("Delete all graph data older than")
 
         selector = hildon.TouchSelector(text = True)
-        selector.append_text('3 days')
-        selector.append_text('7 days')
+
+        selector.append_text('90 days')
+        selector.append_text('60 days')
+        selector.append_text('30 days')
         selector.append_text('14 days')
-        selector.append_text('28 days')
+        selector.append_text('7 days')
+        selector.append_text('3 days')
         selector.append_text('Delete all')
         selector.connect("changed", self.doDelete, dlg)
         dlg.vbox.pack_start(selector, True, True)
+        oldest = self.dataStorage.getOldestObservationTime()
+        if oldest != None:
+            oldest = ((time.time() - oldest) / 86400) + 1
+            dlg.vbox.pack_start(gtk.Label('Oldest entry in database is %d days old' % int(oldest)), False, True, 10)
+        else:
+            dlg.vbox.pack_start(gtk.Label('There are no entries in the database'), False, True, 10)
         dlg.vbox.set_size_request(300, 300)
         dlg.set_transient_for(self)
 
@@ -162,7 +239,9 @@ class MainWindow(hildon.StackableWindow):
             values = {'3 days': now - 3 * 86400,
                       '7 days': now - 7 * 86400,
                       '14 days': now - 14 * 86400,
-                      '28 days': now - 28 * 86400,
+                      '30 days': now - 30 * 86400,
+                      '60 days': now - 60 * 86400,
+                      '90 days': now - 90 * 86400,
                       'Delete all': now + 1}
             self.dataStorage.deleteObservations(values[selector.get_current_text()])
             self.graph.update()
@@ -195,15 +274,15 @@ class MainWindow(hildon.StackableWindow):
         try:
             graphEnd = self.graph.xMax
         except:
-            print "Not initialized yet..."
+            #print "Not initialized yet..."
             return
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
         now = int(time.time())
         if graphEnd < now + 3600:
-            print "Queuing full update"
+            #print "Queuing full update"
             gobject.idle_add(self.doUpdate)
         else:
-            print "Queuing incremental update"
+            #print "Queuing incremental update"
             gobject.idle_add(self.doUpdateInc)
         
     def doUpdateInc(self):
@@ -234,10 +313,10 @@ class MainWindow(hildon.StackableWindow):
             return
 
         if self.get_is_topmost():
-            print "I'm topmost, updating now"
+            print "I'm visible, updating now"
             self.queueUpdate()
         else:
-            print "I'm not topmost, updating later"
+            print "I'm not visible, updating later"
             self.updateRequired = True
         
     def quit(self):
@@ -297,7 +376,7 @@ class EraseableBuffer(object):
         return self.pixmap.get_size()
 
 class Graph(hildon.PannableArea):
-    def __init__(self, dataStorage):
+    def __init__(self, dataStorage, colorScheme):
         hildon.PannableArea.__init__(self)
         self.dataStorage = dataStorage
         self.graphWidth = 800
@@ -315,28 +394,19 @@ class Graph(hildon.PannableArea):
 
         self.get_hadjustment().connect("value-changed", self.moveLabels)
 
-        self.bgColor = self.area.get_colormap().alloc_color(0, 0, 0)
-        self.gridColor = self.area.get_colormap().alloc_color(40*256, 40*256, 45*256)
-        self.labelsColor = self.area.get_colormap().alloc_color(120*256, 120*256, 120*256)
-
-        self.dataColors = {PERCENTAGE: self.area.get_colormap().alloc_color(0, 255*256, 0),
-                           CHARGE: self.area.get_colormap().alloc_color(0, 100*256, 20*256),
-                           VOLTAGE: self.area.get_colormap().alloc_color(60*256, 60*256, 100*256)}
-
-        self.chargingColor = self.area.get_colormap().alloc_color(35*256, 35*256, 0)
-        self.fullColor = self.area.get_colormap().alloc_color(0, 50*256, 0)
-
         self.datasets = {PERCENTAGE:
                             bool(int(self.dataStorage.getConfig('display:' + PERCENTAGE, 1))),
                          CHARGE:
-                            bool(int(self.dataStorage.getConfig('display:' + CHARGE, 0))),
+                            bool(int(self.dataStorage.getConfig('display:' + CHARGE, 1))),
                          VOLTAGE:
-                            bool(int(self.dataStorage.getConfig('display:' + VOLTAGE, 0)))}
+                            bool(int(self.dataStorage.getConfig('display:' + VOLTAGE, 1)))}
         
         self.dataRenderers = {VOLTAGE: [],
                               CHARGE: [],
                               PERCENTAGE: []}
 
+        self.setColorScheme(colorScheme)
+
         self.unrenderedArea = SliceSet()
 
         self.chargeInfo = []
@@ -348,6 +418,21 @@ class Graph(hildon.PannableArea):
 
         self.requestJump = None
 
+    def setColorScheme(self, colorScheme):
+        cm = self.area.get_colormap()
+        self.colors = {}
+        for key in ('background', 'grid', 'x_labels'):
+            self.colors[key] = cm.alloc_color(*(i*256 for i in colorScheme[key]))
+
+        self.dataColors = {}
+        for key, valuebase in {PERCENTAGE: 'data_percentage_',
+                               CHARGE: 'data_charge_',
+                               VOLTAGE: 'data_voltage_'}.iteritems():
+            self.dataColors[key] = (cm.alloc_color(*(i*256 for i in colorScheme[valuebase + '1'])),
+                                    cm.alloc_color(*(i*256 for i in colorScheme[valuebase + '2'])))
+        self.dataColors['charging'] = cm.alloc_color(*(i*256 for i in colorScheme['data_charging']))
+        self.dataColors['full'] = cm.alloc_color(*(i*256 for i in colorScheme['data_full']))
+
     def update(self):
         self.dataRenderers = {VOLTAGE: [],
                               CHARGE: [],
@@ -355,17 +440,14 @@ class Graph(hildon.PannableArea):
         self.chargeInfo = []
         self.updateGrid()
         self.updateData()
-        #self.renderData(self.drawAreaInfo.canvasAsRect())
         self.requestJump = self.drawAreaInfo.canvasSize[0]-1
         self.area.queue_draw()
-        #gobject.idle_add(self.moveLabels)
 
     def updateIncremental(self):
         self.updateDataIncremental()
-        self.unrenderedArea = SliceSet([(0, self.drawAreaInfo.canvasSize[0] - 1)])
+        invalidate = self.getInvalidateableRangeOnIncrementalUpdate()
+        self.unrenderedArea.add((invalidate[0], invalidate[1]))
         self.area.queue_draw()
-        #self.renderData(self.drawAreaInfo.canvasAsRect())
-        #gobject.idle_add(self.moveLabels)
 
     def enableDataset(self, dataset):
         #print "Enable: %s" % dataset
@@ -375,8 +457,6 @@ class Graph(hildon.PannableArea):
         self.changeYLabels(dataset)
         self.unrenderedArea = SliceSet([(0, self.drawAreaInfo.canvasSize[0] - 1)])
         self.area.queue_draw()
-        #self.renderData(self.drawAreaInfo.canvasAsRect())
-        #gobject.idle_add(self.moveLabels)
 
     def disableDataset(self, dataset):
         #print "Disable: %s" % dataset
@@ -386,8 +466,6 @@ class Graph(hildon.PannableArea):
             self.changeYLabels(self.getBestYLabelType())
         self.unrenderedArea = SliceSet([(0, self.drawAreaInfo.canvasSize[0] - 1)])
         self.area.queue_draw()
-        #self.renderData(self.drawAreaInfo.canvasAsRect())
-        #gobject.idle_add(self.moveLabels)
 
     def changeVisibleHours(self, visibleHours):
         #print "Visible Hours: %d" % visibleHours
@@ -400,12 +478,24 @@ class Graph(hildon.PannableArea):
                 r.changeScale(self.drawAreaInfo, timeBounds)
         for r in self.chargeInfo:
             r.changeScale(self.drawAreaInfo, timeBounds)
-        #self.renderData(self.drawAreaInfo.canvasAsRect())
         self.requestJump = self.drawAreaInfo.canvasSize[0]-1
         self.area.queue_draw()
-        #gobject.idle_add(self.moveLabels)
         
 
+    def getInvalidateableRangeOnIncrementalUpdate(self):
+        lastRenderers = []
+        for rendererList in (self.chargeInfo, self.dataRenderers[PERCENTAGE], self.dataRenderers[CHARGE], self.dataRenderers[VOLTAGE]):
+            try:
+                lastRenderers.append(rendererList[-1])
+            except IndexError:
+                pass
+        minX = 0
+        for r in lastRenderers:
+            cb = r.getCoordBounds()
+            if cb[1] > cb[0]:
+                minX = min(minX, cb[0])
+        return (minX, self.drawAreaInfo.canvasSize[0] - 1)
+
     def getBestYLabelType(self):
         for s in (PERCENTAGE, CHARGE, VOLTAGE):
             if self.datasets[s]:
@@ -435,7 +525,7 @@ class Graph(hildon.PannableArea):
         return ((yMin, yMax), yLabelsInterval, yLabels)
      
     def changeYLabels(self, type):
-        self.grid.changeYGrid(*(self.getYLabelInfo(type) + (self.dataColors[type], )))
+        self.grid.changeYGrid(*(self.getYLabelInfo(type) + (self.dataColors[type][0], self.colors['background'])))
         self.gridType = type
         self.dataStorage.setConfig('display:gridType', type)
 
@@ -481,8 +571,9 @@ class Graph(hildon.PannableArea):
         self.grid = GridRenderer(self.drawAreaInfo,
                                  timeBounds, self.xGridInterval,
                                  valueBounds, yLabelsInterval,
-                                 self.xLabelInterval, self.labelsColor,
-                                 yLabels, self.dataColors[labelsType],
+                                 self.xLabelInterval, self.colors['x_labels'],
+                                 yLabels, self.dataColors[labelsType][0],
+                                 self.colors['background'],
                                  self.area.create_pango_context())
 
         self.gridType = labelsType
@@ -549,7 +640,7 @@ class Graph(hildon.PannableArea):
     def renderData(self, cliprect):
         gc = self.pixmap.new_gc()
         gc.set_clip_rectangle(cliprect)
-        gc.foreground = self.bgColor
+        gc.foreground = self.colors['background']
 
         area = self.grid.clearYLabels(self.pixmap)
         if area:
@@ -561,10 +652,10 @@ class Graph(hildon.PannableArea):
                                    self.drawAreaInfo.canvasSize[1])
         
         for d in self.chargeInfo:
-            d.render(self.pixmap, {(True, True): self.fullColor, (True, False): self.chargingColor},
+            d.render(self.pixmap, {(True, True): self.dataColors['full'], (True, False): self.dataColors['charging']},
                      cliprect)
 
-        self.grid.render(self.pixmap, self.gridColor, cliprect)
+        self.grid.render(self.pixmap, self.colors['grid'], cliprect)
 
         for type, renderers in ((t, self.dataRenderers[t])
                                 for t in (VOLTAGE,
@@ -605,8 +696,16 @@ class Graph(hildon.PannableArea):
             x = min(edges)
             w = max(edges) - min(edges) + 1
             r = gtk.gdk.Rectangle(x, 0, w, self.drawAreaInfo.canvasSize[1])
-            #print "Render (%d, %d)->(%d, %d)" % (r.x, r.y, r.x + r.width, r.y + r.height)
-            self.unrenderedArea.subtract((x, max(edges)))
+
+            # Clip rectangle x-axis expansion (so that areas between breaks only need
+            # to be drawn once, even when they would be larger than the clip rectangle)
+            for renderer in self.chargeInfo + self.dataRenderers[VOLTAGE] + self.dataRenderers[CHARGE] + self.dataRenderers[PERCENTAGE]:
+                r = renderer.expandCliprect(r)
+#                print "Renderer %s, (%d -> %d)" % (str(type(renderer)), renderer.getCoordBounds()[0], renderer.getCoordBounds()[1])
+
+            #print "%d segments" % len(self.chargeInfo)
+            #print "Render (%d, %d)->(%d, %d)" % (r.x, r.y, r.x + r.width - 1, r.y + r.height - 1)
+            self.unrenderedArea.subtract((r.x, r.x + r.width - 1))
             gobject.idle_add(self.renderData, r)
         else:
             widget.window.draw_drawable(self.gc, self.pixmap,
@@ -686,7 +785,99 @@ class Graph(hildon.PannableArea):
 #                                                   datetime.datetime.fromtimestamp(status[1], beye.utc).astimezone(beye.local),
 #                                                   datetime.datetime.fromtimestamp(status[2], beye.utc).astimezone(beye.local))
 
-  
+class SettingsItem(object):
+    def __init__(self, *args):
+        if len(args) == 1 and len(args[0]) == 2:
+            self.content = args[0]
+        elif len(args) == 2:
+            self.content = (key, description)
+        else:
+            raise ArgumentException('Don\'t know what to do with constructor argument.')
+    def __repr__(self):
+        return "<%s: %s>" % self.content
+    def __eq__(self, other):
+        if isinstance(other, type(self)):
+            return self == other
+        return self.content[0] == other
+    def __getitem__(self, item):
+        if item in (0, 1):
+            return self.content[item]
+        raise IndexError
+
+class SettingsDialog(gtk.Dialog):
+    def __init__(self, parent):
+        gtk.Dialog.__init__(self, title='Settings', parent=parent, buttons=('Save', gtk.RESPONSE_OK))
+
+        self.set_transient_for(parent)
+
+        self.choices = {}
+        self.selected = {}
+        self.choices['daemon:autoDelete'] = map(SettingsItem,
+                                         [(0, 'off'),
+                                          (86400 * 90, '90 days'),
+                                          (86400 * 60, '60 days'),
+                                          (86400 * 30, '30 days'),
+                                          (86400 * 14, '14 days'),
+                                          (86400 * 7, '7 days'),
+                                          (86400 * 3, '3 days')])
+        self.selected['daemon:autoDelete'] = int(parent.dataStorage.getConfig('daemon:autoDelete', 0))
+        self.choices['display:colorScheme'] = map(SettingsItem, ((k, k) for k in colorSchemes.keys()))
+        self.selected['display:colorScheme'] = parent.dataStorage.getConfig('display:colorScheme', 'Default')
+
+    def _updateSelected(self, selector, data, key):
+        self.selected[key] = self.choices[key][selector.get_active(0)][0]
+
+    def _createSelectorButton(self, title, key):
+        button = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        button.set_title(title)
+        selector = hildon.TouchSelector(text = True)
+        for value,text in (self.choices[key]):
+            selector.append_text(text)
+        try:
+            selector.set_active(0, self.choices[key].index(self.selected[key]))
+        except ValueError:
+            selector.set_active(0, 0)
+        selector.connect("changed", self._updateSelected, key)
+        button.set_selector(selector)
+        button.set_alignment(0, 1, 0, 0)
+        return button
+
+    def showSettings(self, mainWindow):
+
+        self.colorScheme = self._createSelectorButton('Color scheme', 'display:colorScheme')
+        self.vbox.pack_start(self.colorScheme, False, False, 10)
+
+        self.autodelete = self._createSelectorButton('Automatically delete old data', 'daemon:autoDelete')
+        self.vbox.pack_start(self.autodelete, False)
+        
+        stat = os.stat(beye.dbFilePath)
+        self.dbSizeLabel = gtk.Label('Database size: %dkB, %d records' % (stat[6] / 1024, mainWindow.dataStorage.countObservations()))
+        oldest = mainWindow.dataStorage.getOldestObservationTime()
+        if oldest != None:
+            oldest = ((time.time() - oldest) / 86400) + 1
+            self.dbAgeLabel = gtk.Label('Oldest entry in database: %d days old' % int(oldest))
+        else:
+            self.dbAgeLabel = gtk.Label('There are no entries in the database')
+        self.dbSizeLabel.set_alignment(0, 0)
+        self.dbAgeLabel.set_alignment(0, 0)
+
+        self.vbox.pack_start(self.dbSizeLabel, False)
+        self.vbox.pack_start(self.dbAgeLabel, False, True)
+        
+        self.vbox.set_size_request(300, 300)
+
+        self.show_all()
+        return self.run()
+
+    def applySettings(self, mainWindow):
+        ds = mainWindow.dataStorage
+        for key, value in self.selected.iteritems():
+            print "%s: %s" % (key, value)
+            mainWindow.dataStorage.setConfig(key, value)
+        mainWindow.colorScheme = colorSchemes[self.selected['display:colorScheme']]
+        mainWindow.graph.setColorScheme(mainWindow.colorScheme)
+        mainWindow.graph.update()
+
 class AboutDialog(gtk.AboutDialog):
     def __init__(self, parent):
         gtk.Dialog.__init__(self)
@@ -697,9 +888,6 @@ class AboutDialog(gtk.AboutDialog):
         self.set_version(beye.__version__)
         self.set_website(beye.__url__)
 
-        stat = os.stat(beye.dbFilePath)
-        self.set_comments('Database size: %dkB, %d records' % (stat[6] / 1024, parent.dataStorage.countObservations()))
-
         #gtk.about_dialog_set_url_hook(lambda dialog, link, user_data: webbrowser.open(link), None)
         self.set_license('''The battery-eye software is distributed under GPL version 3. For more information, see
 <http://www.gnu.org/licenses/>
@@ -718,7 +906,7 @@ battery-eye is distributed in the hope that it will be useful, but WITHOUT ANY W
 
 class GridRenderer(object):
     def __init__(self, areaInfo, timeBounds, xInterval, valueBounds, yInterval,
-                 xLabelInterval, xLabelsColor, yLabels, yLabelsColor, pangoContext):
+                 xLabelInterval, xLabelsColor, yLabels, yLabelsColor, bgcolor, pangoContext):
         self.areaInfo = areaInfo
         self.timeBounds = timeBounds
         self.valueBounds = valueBounds
@@ -732,15 +920,15 @@ class GridRenderer(object):
         self.yLabelsColor = yLabelsColor
         self.xLabelPixmaps = {}
         self.pangoContext = pangoContext
-        self.yLabels = self.createYLabels(yLabels, yLabelsColor)
-        self.xLabels = self.createXLabels(timeBounds, xLabelInterval, xLabelsColor)
+        self.yLabels = self.createYLabels(yLabels, yLabelsColor, bgcolor)
+        self.xLabels = self.createXLabels(timeBounds, xLabelInterval, xLabelsColor, bgcolor)
 
-    def changeYGrid(self, bounds, interval, labels, color):
+    def changeYGrid(self, bounds, interval, labels, color, bgcolor):
         self.valueBounds = bounds
         self.yInterval = interval
         self.yLabelsColor = color
         self.yGrid = self.yGridSegments(self.areaInfo, bounds, interval)
-        self.yLabels = self.createYLabels(labels, color)
+        self.yLabels = self.createYLabels(labels, color, bgcolor)
 
     def yGridSegments(self, areaInfo, yBounds, yInterval):
         y = []
@@ -755,7 +943,7 @@ class GridRenderer(object):
         points = [time.mktime(h.timetuple()) for h in self.hours]
         return self.createSegments('x', points, areaInfo, xBounds)
 
-    def createXLabels(self, xBounds, modulo, color):
+    def createXLabels(self, xBounds, modulo, color, bgcolor):
         pContext = self.pangoContext
         pContext.set_font_description(pango.FontDescription('sans normal 10'))
         
@@ -775,6 +963,7 @@ class GridRenderer(object):
                 pixmap = gtk.gdk.Pixmap(None, layout.get_pixel_size()[0],
                                               layout.get_pixel_size()[1], 16)
                 gc = pixmap.new_gc()
+                gc.foreground = bgcolor
                 pixmap.draw_rectangle(gc, True, 0, 0, pixmap.get_size()[0], pixmap.get_size()[1])
                 gc.foreground = color
                 pixmap.draw_layout(gc, 0, 0, layout)
@@ -790,7 +979,7 @@ class GridRenderer(object):
                                    x - int(pixmap.get_size()[0]/2), y,
                                    -1, -1)
 
-    def createYLabels(self, labels, color):
+    def createYLabels(self, labels, color, bgcolor):
         pContext = self.pangoContext
         pContext.set_font_description(pango.FontDescription('sans normal 10'))
         labelPixmaps = []
@@ -800,6 +989,7 @@ class GridRenderer(object):
             pixmap = gtk.gdk.Pixmap(None, layout.get_pixel_size()[0],
                                           layout.get_pixel_size()[1], 16)
             gc = pixmap.new_gc()
+            gc.foreground = bgcolor
             pixmap.draw_rectangle(gc, True, 0, 0, pixmap.get_size()[0], pixmap.get_size()[1])
             gc.foreground = color
             pixmap.draw_layout(gc, 0, 0, layout)
@@ -862,10 +1052,47 @@ class GridRenderer(object):
         gc.set_clip_rectangle(cliprect)
         drawable.draw_segments(gc, self.yGrid)
         drawable.draw_segments(gc, self.xGrid)
+        gc = drawable.new_gc()
+        gc.set_clip_rectangle(cliprect)
         self.drawXLabels(gc, drawable, self.xLabels)
-                            
-class DataRendererValue(object):
+
+class DataRenderer(object):
+    def __init__(self):
+        self.coordBounds = (0, 0)
+
+    def getCoordBounds(self):
+        return self.coordBounds
+
+    def updateCoordBounds(self):
+        try:
+            first = self.scaledData[0][0]
+            last = self.scaledData[-1][0]
+            # X coordinates can be either a single integer, or a two-integer range
+            if not type(first) == int:
+                first = first[0]
+            if not type(last) == int:
+                last = last[-1]
+            self.coordBounds = (first, last)
+        except IndexError:
+            # There is no data
+            self.coordBounds = (0, 0)
+
+    def expandCliprect(self, cliprect):
+        if max(cliprect.x, self.coordBounds[0]) <= min(cliprect.x + cliprect.width - 1, self.coordBounds[1]) \
+           and (cliprect.x > self.coordBounds[0] or cliprect.x + cliprect.width - 1 < self.coordBounds[1]):
+            # Expanding the clip rectangle x size to match the current renderer
+            rect = gtk.gdk.Rectangle(min(cliprect.x, self.coordBounds[0]),
+                                     cliprect.y,
+                                     max(cliprect.x + cliprect.width, self.coordBounds[1]) - min(cliprect.x, self.coordBounds[0]),
+                                     cliprect.height)
+#            print "Expanding cliprect, (%d, %d) -> (%d, %d)" % (cliprect.x, cliprect.x + cliprect.width - 1,
+#                                                                rect.x, rect.x + rect.width - 1)
+            return rect
+        return cliprect
+
+class DataRendererValue(DataRenderer):
     def __init__(self, rawData, areaInfo, timeBounds, valueBounds):
+        DataRenderer.__init__(self)
         self.rawData = rawData
         self.areaInfo = areaInfo
         self.timeBounds = timeBounds
@@ -873,13 +1100,6 @@ class DataRendererValue(object):
         self.scaledData = self.rescaleData(self.rawData, self.areaInfo, self.timeBounds, self.valueBounds)
         self.updateCoordBounds()
 
-    def updateCoordBounds(self):
-        try:
-            self.coordBounds = (self.scaledData[0][0]-1, self.scaledData[-1][0]+1)
-        except IndexError:
-            # There is no data
-            self.coordBounds = (-1, -1)
-
     def getLastObservationTime(self):
         if len(self.rawData):
             return self.rawData[-1][0]
@@ -916,16 +1136,13 @@ class DataRendererValue(object):
         return [(int(scaleX * (p[0] - xBounds[0])) + offX,
                  int(scaleY * (p[1] - yBounds[0])) + dispY) for p in points]
 
-    def render(self, drawable, color, cliprect):
+    def render(self, drawable, colors, cliprect):
         if max(cliprect.x, self.coordBounds[0]) > min(cliprect.x + cliprect.width - 1, self.coordBounds[1]):
             # Nothing to see here, move along.
             return
-        gc = drawable.new_gc(foreground=color, line_width=1, join_style=gtk.gdk.JOIN_ROUND)
+        gc = drawable.new_gc(foreground=colors[0], line_width=1, join_style=gtk.gdk.JOIN_ROUND)
         gc.set_clip_rectangle(cliprect)
-        color2 = drawable.get_colormap().alloc_color(int(color.red*0.5),
-                                                     int(color.green*0.5),
-                                                     int(color.blue*0.5))
-        gc2 = drawable.new_gc(foreground=color2, line_width=2, join_style=gtk.gdk.JOIN_ROUND)
+        gc2 = drawable.new_gc(foreground=colors[1], line_width=2, join_style=gtk.gdk.JOIN_ROUND)
         gc2.set_clip_rectangle(cliprect)
 
         if len(self.scaledData) == 1:
@@ -935,21 +1152,15 @@ class DataRendererValue(object):
             drawable.draw_lines(gc2, self.scaledData)
             drawable.draw_lines(gc, self.scaledData)
 
-class DataRendererStatus(object):
+class DataRendererStatus(DataRenderer):
     def __init__(self, rawData, areaInfo, timeBounds):
+        DataRenderer.__init__(self)
         self.rawData = rawData
         self.areaInfo = areaInfo
         self.timeBounds = timeBounds
         self.scaledData = self.rescaleData(self.rawData, self.areaInfo, self.timeBounds)
         self.updateCoordBounds()
 
-    def updateCoordBounds(self):
-        try:
-            self.coordBounds = (self.scaledData[0][1], self.scaledData[-1][2])
-        except IndexError:
-            # There is no data
-            self.coordBounds = (-1, -1)
-
     def getFirstObservationTime(self):
         if not len(self.rawData):
             return None
@@ -973,9 +1184,9 @@ class DataRendererStatus(object):
 
         offX = self.areaInfo.offset[0]
 
-        return [(s[0],
-                 int(scaleX * (s[1] - xBounds[0])) + offX,
-                 int(scaleX * (s[2] - xBounds[0])) + offX) for s in statuses]
+        return [((int(scaleX * (s[1] - xBounds[0])) + offX,
+                  int(scaleX * (s[2] - xBounds[0])) + offX),
+                 s[0]) for s in statuses]
 
     def render(self, drawable, colorMap, cliprect):
         if max(cliprect.x, self.coordBounds[0]) > min(cliprect.x + cliprect.width - 1, self.coordBounds[1]):
@@ -983,13 +1194,13 @@ class DataRendererStatus(object):
             return
         #print "render -> %d - %d" % self.coordBounds
         for status in self.scaledData:
-            if (not colorMap.has_key(status[0])) or colorMap[status[0]] == None:
+            if (not colorMap.has_key(status[1])) or colorMap[status[1]] == None:
                 continue
-            gc = drawable.new_gc(foreground=colorMap[status[0]])
+            gc = drawable.new_gc(foreground=colorMap[status[1]])
             gc.set_clip_rectangle(cliprect)
             drawable.draw_rectangle(gc, True,
-                                    status[1], self.areaInfo.offset[1],
-                                    status[2] - status[1], self.areaInfo.offset[1] + self.areaInfo.drawAreaSize[1] - 14)
+                                    status[0][0], self.areaInfo.offset[1],
+                                    status[0][1] - status[0][0], self.areaInfo.offset[1] + self.areaInfo.drawAreaSize[1] - 14)
 
 
 def getHoursInInterval(start, end, modulo):
